#Created by Ryan Morrison 6-30-17
#File to create python GUI
#Created with help from:
#http://effbot.org/tkinterbook/frame.htm, http://effbot.org/tkinterbook/grid.htm

####### 4 Frame setup:
# Column 1 Column 2 Column 3
# --------------
# Row 0
# Row 1                     <- Frame 1 (1x1 grid)
# Row 2                
# Row 3
# --------------
# Row 0  Row 0   Row 0
# Row 1  Row 1   Row 1
# Row 2  Row 2   Row 2
# Row 3  Row 3   Row 3      <- Frame 2 (3x8 grid)
# Row 4  Row 4   Row 4
# Row 5  Row 5   Row 5
# Row 6  Row 6   Row 6
# Row 7  Row 7   Row 7
# --------------
#       button              <- Frame 3 (no grid)
#---------------
# Row 0 Row 0 Row 0 
# Row 1 Row 1 Row 1         <- Frame 4 (3x2 grid)

from tkinter import *
root = Tk()
oxygen_percent_string = StringVar()
temperature_string = StringVar()
flow_rate_string = StringVar()
override_checkbox_int = IntVar()
duty_cycle_var = DoubleVar()
interval_var = DoubleVar()

hasStarted = False

#function to delay 
def start():
    global hasStarted
    if (hasStarted == False):
        hasStarted = True

def setup_GUI():
    #Configure the GUI
    #delay before the start button is pressed
    global hasStarted
    gray_blue = '#%02x%02x%02x' % (29, 32, 37) #(r,g,b)
    light_gray_blue = '#%02x%02x%02x' % (29*2, 32*2, 37*2)
    light_light_gray_blue = '#%02x%02x%02x' % (29*3, 32*3, 37*3)
    light_blue = '#%02x%02x%02x' % (52, 121, 152)
    medium_blue = '#%02x%02x%02x' % (36, 102, 134)
    dark_blue = '#%02x%02x%02x' % (50, 91, 125)
    green = '#%02x%02x%02x' % (34, 168, 108)
    orange = '#%02x%02x%02x' % (232, 115, 82)
    yellow = '#%02x%02x%02x' % (235, 201, 94)

    root.configure(bg=gray_blue)
    root.minsize(1000, 800)
    root.wm_title("Oxygen Controller GUI")
    oxygen_var = DoubleVar()
    frequency_var = DoubleVar()

    oxygen_p_var1 = IntVar()
    oxygen_p_var2 = IntVar()

    time1_var = DoubleVar()
    time2_var = DoubleVar()
    total_runtime_var = DoubleVar()

    osc_p_var1 = IntVar()
    osc_p_var2 = IntVar()

    #Four frames for four separate grids
    text_output_frame = Frame(root,bg=gray_blue)
    grid_input_frame = Frame(root,bg=light_light_gray_blue)
    button_frame = Frame(root,bg=gray_blue)
    override_control_frame = Frame(root,bg=light_gray_blue)
    

    

    oxygen_flow_sensor = Label(text_output_frame, textvar=oxygen_percent_string, font=("Helvetica", 50), bg=gray_blue, fg=green)
    oxygen_flow_sensor.grid(row = 0, sticky = W, padx=50, pady = 20)

    temperature_sensor = Label(text_output_frame, textvar=temperature_string,font=("Helvetica", 50),bg=gray_blue, fg=orange)
    temperature_sensor.grid(row = 1, sticky = W, padx=50, pady = 20)

    flow_rate_sensor = Label(text_output_frame, textvar=flow_rate_string,font=("Helvetica", 50),bg=gray_blue, fg=yellow)
    flow_rate_sensor.grid(row = 2, sticky = W, padx=50, pady = 30)

    program_title = Label(text_output_frame, text="Program to Run:",font=("Helvetica", 40),bg=gray_blue, fg='white')
    program_title.grid(row = 3, sticky = W, padx=10,pady = 10)


    oxygen_percentage_title = Label(grid_input_frame, text="Oxygen Percentage (% O2)",font=("Helvetica", 20,"bold"),bg=light_light_gray_blue, fg='blue2')
    oxygen_percentage_title.grid(row = 0, column = 0, padx=10,pady = 10)

    time_title = Label(grid_input_frame, text="      Time (minutes)       ",font=("Helvetica", 20,"bold"),bg=light_light_gray_blue, fg='blue2')
    time_title.grid(row = 0, column = 1, padx=10,pady = 10)

    oscillation_frequency_title = Label(grid_input_frame, text="Output frequency (mHz)",font=("Helvetica", 20,"bold"),bg=light_light_gray_blue, fg='blue2')
    oscillation_frequency_title.grid(row = 0, column = 2, padx=10,pady = 10)



    oxygen_p_entry1 = Entry(grid_input_frame, textvariable = oxygen_p_var1,font=("Helvetica", 20), width = 15)
    oxygen_p_entry1.grid(column = 0, row = 5)

    time_entry1 = Entry(grid_input_frame, textvariable = time1_var,font=("Helvetica", 20), width = 15)
    time_entry1.grid(column = 1, row = 5)

    oscillation_entry1 = Entry(grid_input_frame, textvariable = osc_p_var1,font=("Helvetica", 20), width = 15)
    oscillation_entry1.grid(column = 2, row = 5)

    oxygen_p_entry2 = Entry(grid_input_frame, textvariable = oxygen_p_var2,font=("Helvetica", 20), width = 15)
    oxygen_p_entry2.grid(column = 0, row = 6)

    time_entry2 = Entry(grid_input_frame, textvariable = time2_var,font=("Helvetica", 20), width = 15)
    time_entry2.grid(column = 1, row = 6)

    oscillation_entry2 = Entry(grid_input_frame, textvariable = osc_p_var2,font=("Helvetica", 20), width = 15)
    oscillation_entry2.grid(column = 2, row = 6)


    total_runtime_title = Label(grid_input_frame, text="Total Runtime (minutes):",font=("Helvetica", 20),bg=light_light_gray_blue, fg='red')
    total_runtime_title.grid(row = 7, column = 0, pady = 10)

    total_runtime_entry = Entry(grid_input_frame, textvariable = total_runtime_var,font=("Helvetica", 20), width = 10)
    total_runtime_entry.grid(column = 1, row = 7, pady = 10)

    start_button = Button(button_frame, text="BEGIN", command=start, height=3, width=10, bg='white',fg='black')
    start_button.grid(row = 0,pady = 20)


    duty_cycle_title = Label(override_control_frame, text="Duty Cycle (%)",font=("Helvetica", 20,"bold"),bg=light_gray_blue, fg=light_blue)
    duty_cycle_title.grid(row = 0, column = 0, padx=10,pady = 10)

    interval_title = Label(override_control_frame, text="Interval (seconds)",font=("Helvetica", 20,"bold"),bg=light_gray_blue, fg=light_blue)
    interval_title.grid(row = 0, column = 1, padx=10,pady = 10)

    override_ON_checkBox = Checkbutton(override_control_frame, variable=override_checkbox_int, onvalue=1, offvalue=0, text="Override Controls ON")
    override_ON_checkBox.grid(row = 1, column = 2 ,pady = 10)

    duty_cycle_entry = Entry(override_control_frame, textvariable = duty_cycle_var,font=("Helvetica", 20), width = 10)
    duty_cycle_entry.grid(column = 0, row = 1, pady = 10)

    interval_entry = Entry(override_control_frame, textvariable = interval_var,font=("Helvetica", 20), width = 10)
    interval_entry.grid(column = 1, row = 1, pady = 10)
    


    text_output_frame.pack(anchor="w")
    grid_input_frame.pack()
    button_frame.pack()
    override_control_frame.pack()


    #update the sensor values displayed on the GUI
    oxygen_percent_string.set("Oxygen Sensor: " + "--" + "% Oxygen")
    temperature_string.set("Temperature: " + "--" + " " + u'\u2103') #degrees C
    flow_rate_string.set("Flow Rate: " + "--" + " L/min")


    #delay before the start button is pressed
    while (hasStarted == False):
        root.update()

    variable_array = []
    variable_array.append(oxygen_p_var1.get())
    variable_array.append(oxygen_p_var2.get())
    variable_array.append(time1_var.get())
    variable_array.append(time2_var.get())
    variable_array.append(osc_p_var1.get())
    variable_array.append(osc_p_var2.get())
    variable_array.append(total_runtime_var.get())

    return variable_array


def update_GUI(oxygen,temperature,flow_rate):
    oxygen_percent_string.set("Oxygen Sensor: " + str(oxygen) + " % Oxygen")
    temperature_string.set("Temperature: " + str(temperature) + " "+ u'\u2103')
    flow_rate_string.set("Flow Rate: " + str(flow_rate) + " L/min")            
    root.update()

def get_Arduino_1_GUI_values():
    return duty_cycle_var.get(),interval_var.get(),override_checkbox_int.get()

