#Created by Ryan Morrison 6-30-17
#Script to talk with EOM-O2-FDM-SMA sesnor and two arduinos.
#Two log files are created when program is run. A graphical
#user interface takes input from a user.

import time
import serial
import io
import struct
import os
import sys
import datetime
import GUI

automatic_temperature_control_ON = False

#Stores flow rate data to text with date and time
def write_to_flow_text_file(full_path,measurement_number,flow_rate):
    log_file_array = []
    #get local date and time
    now = datetime.datetime.now()
    date = now.strftime("%m/%d/%y")
    time = now.strftime("%H:%M:%S")
    #print to log file
    log_file_array.append(date)
    log_file_array.append(time)
    log_file_array.append(str(flow_rate))
    with open(full_path, 'a') as f:
        for i in range(0,(len(log_file_array)-1)):
            f.write(log_file_array[i] + ",")
        f.write(log_file_array[(len(log_file_array)-1)])
        f.write("\n")
    del log_file_array[:]

#Stores oxygen sensor data to text with date and time
def write_to_oxygen_text_file(full_path,measurement_number,gui_value,sensor_value,temperature):
    log_file_array = []
    #get local date and time
    now = datetime.datetime.now()
    date = now.strftime("%m/%d/%y")
    time = now.strftime("%H:%M:%S")
    #print to log file
    log_file_array.append(date)
    log_file_array.append(time)
    log_file_array.append(str(measurement_number))
    log_file_array.append(str(gui_value))
    log_file_array.append(str(sensor_value))
    log_file_array.append(str(temperature))
    with open(full_path, 'a') as f:
        for i in range(0,(len(log_file_array)-1)):
            f.write(log_file_array[i] + ",")
        f.write(log_file_array[(len(log_file_array)-1)])
        f.write("\n")
    del log_file_array[:]

def transmit_to_Arduino_1(GUI_set_oxygen_level,oxygen,duty_cycle,interval,automatic_control_status):
    #Send the message to the arduino 1 that controls nitrogen valve
    values_to_send = str(GUI_set_oxygen_level) +','+ str(oxygen)+','+ str(duty_cycle)+','+ str(interval)+','+ str(automatic_control_status)
    print('Sending the following to Arduino 1: ' + values_to_send)                            
    failed_counter = 0
    not_sent = True
    while not_sent:
        ser2.write(values_to_send.encode())
        time.sleep(0.5) #seconds
        acknowledge = ser2.read(ser2.inWaiting())
        #print(acknowledge)
        if acknowledge.decode('utf-8') == "ACK\r\n":
            not_sent = False
            print ("Successfully Transmitted to Arduino 1")
        else:
            failed_counter += 1
            time.sleep(.7) #.5 seconds
            if(failed_counter > 75):
                print ("FAILED to Transmit to Arduino 1")
                sys.exit() 
    

#Initialize the serial communication for 1st OXY-1 Sensor
ser1 = serial.Serial('COM4',19200)
#configure the serial communication for Arduino 1
ser2 = serial.Serial('COM6',9600)
#configure the serial communication for Arduino 2
ser3 = serial.Serial('COM5',9600,timeout=0.1)


#Initialize Two Log Text files with column titles
Current_foler_path = os.path.dirname(os.path.abspath(__file__))
now = datetime.datetime.now()
Oxygen_Sensor_TXT = now.strftime("logs\%m-%d-%y_%H-%M-%S_oxygen_log.txt")
Flow_Sensor_TXT = now.strftime("logs\%m-%d-%y_%H-%M-%S_flow_log.txt")
full_path_oxygen = os.path.join(Current_foler_path,Oxygen_Sensor_TXT)
full_path_flow = os.path.join(Current_foler_path,Flow_Sensor_TXT)
with open(full_path_oxygen, 'a') as f:
        f.write("Date,Time,Measurement Number,GUI Value,Sensor Value,Temperature\n")
with open(full_path_flow, 'a') as f:
        f.write("Date,Time,Measurement Number,Flow Rate\n")






#Attempts to open the serial ports 
try: 
    ser1.open()
    print("Attempting to open ports: " + str(ser1.portstr))
    ser2.open()
    print("Attempting to open ports: " + str(ser2.portstr))
    ser3.open()
    print("Attempting to open ports: " + str(ser3.portstr))

#If there is an exeption raised when trying to open the COM ports,    
except Exception as e:
    print ("Error, " + str(e))




#If the serial ports are open,    
if ser1.is_open and ser2.is_open and ser3.is_open:
    print("Successfully opened: " + str(ser1.portstr) +' and '+ str(ser2.portstr)+' and '+ str(ser3.portstr))

    try:
        print("Arduino's initializing...")
        time.sleep(7)
        print("Arduino's should be done intializing.. trying to connect")
        
        #Wait for Arudino 1 to finish initializing (Arduino will send "Arduino 1 Start")
        not_received = True;
        failed_counter = 0
        while not_received:
            initial_response = ser2.read(ser2.inWaiting())
            if initial_response.decode('utf-8') == "Arduino 1 Start\r\n":
                acknowledge_command = 'ACK'
                ser2.write(acknowledge_command.encode())
                not_received = False
                print ("Successfully recieved Arduino 1 is ready to go, transmitted ACK")
                time.sleep(2)
            else:
                failed_counter += 1
                time.sleep(.2) #.2 seconds
                if(failed_counter > 50):
                    print ("FAILED to receive ready message from Arduino 1")
                    sys.exit()
                
        #Wait for Arudino 2 to finish initializing (Arduino will send "Arduino 2 Start")
        not_received1 = True;
        failed_counter = 0
        while not_received1:
            initial_response1 = ser3.read(ser3.inWaiting())
            if initial_response1.decode('utf-8') == "Arduino 2 Start\r\n":
                acknowledge_command = 'ACK'
                ser3.write(acknowledge_command.encode())
                not_received1 = False
                print ("Successfully recieved Arduino 2 is ready to go, transmitted ACK")
                time.sleep(2)
            else:
                failed_counter += 1
                time.sleep(.1) #.2 seconds
                if(failed_counter > 100):
                    print ("FAILED to receive ready message from Arduino 2")
                    sys.exit()


        #Set Oxygen Measurement Units to %O2 
        oxygen_unit = 1  #set to %O2 (see manual for commands)
        unit_command = 'oxyu000' + str(oxygen_unit) + '\r'
        ser1.write(unit_command.encode())
        time.sleep(2)

        #Set to manual temperature if boolean automatic_temperature_control_ON is False
        if(not automatic_temperature_control_ON):
            temp_command = 'tmpc3700\r'   #change this value if you want something besides 37.00
            ser1.write(temp_command.encode())
            time.sleep(2)
            ser1.write(temp_command.encode())
            time.sleep(2)
            ser1.write(temp_command.encode())
            time.sleep(0.1)
            print("Sent over 37.00 degrees 3 times")
        
        
        #Setup the GUI (separate python file)
        GUI_values = GUI.setup_GUI()
        
        #Get initial values from the GUI 
        GUI_set_oxygen_level = GUI_values[0] 
        GUI_set_osc_freq = GUI_values[4]
        print("Setting oxygen value to: " + str(GUI_values[0]))
        print("Setting oscillation frequency to: " + str(GUI_values[4]))
        #Set the program intervals
        measurement_interval = 30 #every 30 (seconds)
        runtime_interval = GUI_values[6]*60 #total runtime (seconds)
        change_to_2nd_oxygen_p_interval = GUI_values[2]*60 #when to change the values (seconds)
        change_to_1st_oxygen_p_interval = GUI_values[3]*60 #when to change the values (seconds)
        time_to_flip_values = False

        #Initialize counter and timer variables for time keeping 
        current_time = time.time()
        runtime_timer = current_time
        change_to_2nd_oxygen_p_timer = current_time
        change_to_1st_oxygen_p_timer = current_time
        measurement_timer = 0 #take a measurement immediately
        measurement_number_oxygen = 0;
        measurement_number_flow = 0;

        #Discard anything in input serial buffer for arduino 2
        ser3.reset_input_buffer() 


        #The main program while loop (executes until runtime_interval reached) 
        while(current_time - runtime_timer <= runtime_interval):
            
            current_time = time.time() #record current number of seconds elapsed since start of program

            #Take an oxygen measurement every 30 seconds                              
            if (current_time - measurement_timer >= measurement_interval):
                #Send data request
                line = []
                data_command = 'data\r'
                ser1.write(data_command.encode())
                #Read the response from oxygen sensor
                stop = True
                while stop:
                    for c in ser1.read():
                        line.append(chr(c))
                        if chr(c) == '\r':
                            #print(line)
                            stop = False
                #Decode the message from oxygen sensor (read manual for how this is done)
                t_index = line.index('T')
                o_index = line.index('O')
                e_index = line.index('E')
                temperature = float(''.join(line[t_index+1:o_index-3]) + '.' + ''.join(line[o_index-3:o_index-1]))
                if oxygen_unit == 4 or oxygen_unit == 6: #4 decimal places
                    oxygen = float(''.join(line[o_index+1:e_index-5]) + '.' + ''.join(line[e_index-5:e_index-1]))
                else: #2 decimal places
                    oxygen = float(''.join(line[o_index+1:e_index-3]) + '.' + ''.join(line[e_index-3:e_index-1]))
                print('The temperature is: ' + str(temperature) + ' ' + u'\u2103') #unicode for degrees C
                print('The oxygen level is: ' + str(oxygen) + '% Oxygen')
                
                
                #Write values to text file
                measurement_number_oxygen += 1
                write_to_oxygen_text_file(full_path_oxygen, measurement_number_oxygen,GUI_set_oxygen_level,oxygen,temperature)
                
                #Send the message to the arduino 1 that controls nitrogen valve
                duty_cycle, interval, checkbox = GUI.get_Arduino_1_GUI_values()
                transmit_to_Arduino_1(GUI_set_oxygen_level,oxygen,duty_cycle,interval,checkbox)                    

                #Update measuremnt timer
                measurement_timer = current_time


            

            #After very first interval, this runs    
            if(current_time - change_to_2nd_oxygen_p_timer >= change_to_2nd_oxygen_p_interval and time_to_flip_values == False):
                print("Setting value to: " + str(GUI_values[1]))
                GUI_set_oxygen_level = GUI_values[1] #set to second value
                GUI_set_osc_freq = GUI_values[5] #set to second value
                #Send the message to the arduino 1 that controls nitrogen valve
                duty_cycle, interval, checkbox = GUI.get_Arduino_1_GUI_values()
                transmit_to_Arduino_1(GUI_set_oxygen_level,oxygen,duty_cycle,interval,checkbox)
                time_to_flip_values = True
                
                change_to_1st_oxygen_p_timer = current_time #reset the next timer to current time

            #After second interval, this runs    
            if(current_time - change_to_1st_oxygen_p_timer >= change_to_1st_oxygen_p_interval and time_to_flip_values == True):
                print("Setting value to: " + str(GUI_values[0]))
                GUI_set_oxygen_level = GUI_values[0]  #set back to first value
                GUI_set_osc_freq = GUI_values[4] #set back to first value
                #Send the message to the arduino 1 that controls nitrogen valve
                duty_cycle, interval, checkbox = GUI.get_Arduino_1_GUI_values()
                transmit_to_Arduino_1(GUI_set_oxygen_level,oxygen,duty_cycle,interval,checkbox) 
                time_to_flip_values = False
                change_to_2nd_oxygen_p_timer = current_time #reset the next timer to current time
                

            #Waits for any serial COM from arduino 2
            if(ser3.inWaiting() > 0):
                flow_rate = (ser3.readline()).decode('utf-8')
                if flow_rate != "ACK\r\n" and flow_rate != '':
                    flow_rate = (str(flow_rate)).rstrip('\r\n')
                    print("Got flow rate of: " + flow_rate)
                    time.sleep(0.1) #seconds
                    print('Sending the following to arduino 2: ' + str(int(GUI_set_osc_freq)) + ",ACK")
                    ser3.write( ((str(int(GUI_set_osc_freq)))+",ACK" ).encode())
                    measurement_number_flow += 1
                    write_to_flow_text_file(full_path_flow, measurement_number_flow,flow_rate)
                    time.sleep(0.3)

            #constantly update the GUI
            GUI.update_GUI(oxygen,temperature,flow_rate)
                
                
        ###Done with the main while loop ###    
   
        #After the main while loop exits, Close the serial ports                
        ser1.close()
        ser2.close()
        ser3.close()
        print("Closed: " + str(ser1.portstr) +' and '+ str(ser2.portstr)+' and '+ str(ser3.portstr))
        ## Done with Program ##


    #If there is an error message at any point during while loop, strop program and print out error    
    except Exception as e1:
        print ("Error communicating...: " + str(e1))
        ser1.close()
        ser2.close()
        ser3.close()
        print("Closed: " + str(ser1.portstr) +' and '+ str(ser2.portstr)+' and '+ str(ser3.portstr))
